#include <stdio.h>
#include <string.h>
#include <stdlib.h>

  /* Sortiranje rjecnika.
     Varijacija 1: polja w i p se dinamicki (re)alociraju.
     Losija varijanta, koja stalno popravlja polje p, za
     svaku (stvarnu) realokaciju polja w.

     Sortiranje zamjenama pokazivaca u p - izbor ekstrema.

     Funkcija unos vraca status ucitavanja rijeci,
     a broj ucitanih rijeci je globalna varijabla.

     Za ucitavanje koristimo da je poznata maksimalna
     moguca duljina rijeci MAXDULJ = 80.
     Za sve prakticne potrebe to je dovoljno i nema smisla
     raditi pravo dinamicko alociranje za citanje stringa
     nepoznate duljine (v. vjezbe, poglavlje 5.2). */

  /* MAXDULJ = max. duljina pojedine rijeci,
     ukjucivo i nul-znak na kraju stringa. */

#define MAXDULJ 80

  /* Globalno dinamicko polje znakova za cijeli rjecnik.
     Globalna duljina dulj_w sluzi samo za kontrolu. */

char *w = NULL;
unsigned dulj_w = 0u;

  /* Globalno dinamicko polje pokazivaca
     na pojedine rijeci - stringove. */

char **p = NULL;

  /* Globalni stvarni broj rijeci u rjecniku = duljina polja p. */

int broj_rijeci = 0;


  /* Ucitava rijeci i vraca status ucitavanja (0 = nema greske).
     Kraj ucitavanja je prazna rijec. Broj ucitanih rijeci je
     globalna varijabla. */

int unos(void)
{
    char q[MAXDULJ] = "";    /* Buffer za citanje stringa. */
    char *temp_w, **temp_p, *poc_rijeci, *stari_w;
    int status = 0, radi = 1, broj = 0, dulj, i;
    unsigned novi_dulj_w;

    while (radi) {

            /* Citanje stringa u buffer i test greske. */
        if (gets(q) == NULL) {
            status = -1;
            break;
        }

        dulj = strlen(q);

            /* Test na praznu rijec - kraj ucitavanja. */
        if (dulj == 0) break;

            /* Imamo nepraznu novu rijec. */

            /* Prvo zapamti staru adresu od w. */
        stari_w = w;

            /* Realociraj w i p. */
            /* Ako realokacija nije uspjela, vrati do tada
               ucitane w, p i broj rijeci. */
        novi_dulj_w = dulj_w + dulj + 1;
        temp_w = (char *) realloc(w, novi_dulj_w * sizeof(char));
        temp_p = (char **) realloc(p, (broj + 1) * sizeof(char *));
        if ((temp_w == NULL) || (temp_p == NULL)) {
            status = 1;
            break;
        }

            /* Realokacija je uspjela, postavi (mozda) nove w i p. */
        w = temp_w;
        p = temp_p;

            /* Kopiraj novu rijec na kraj polja w. */
        poc_rijeci = w + dulj_w;
        strcpy(poc_rijeci, q);
        dulj_w = novi_dulj_w;

            /* Postavi pokazivac(e) u polju p. */

            /* Ako realokacija promijeni pocetnu adresu od w,
               onda sve ranije pokazivace u p treba popraviti,
               jer pokazuju na pogresne adrese - u starom w. */
            /* Ovo se moze napraviti i na samom kraju citanja,
               (v. ds_v1a.c). */
            /* Zato NIJE dobro realocirati JEDNO polje w,
               vec je BOLJE koristiti posebnu alokaciju za
               svaku pojedinu rijec - varijacija 2, a kolicina
               potrebne memorije je ISTA. */
        if (w != stari_w) {
            temp_w = w;
            for (i = 0; i < broj; ++i) {
                p[i] = temp_w;
                temp_w = temp_w + strlen(temp_w) + 1;
            }
        }

        p[broj] = poc_rijeci;

            /* Kontrolni ispis. */
        printf(" rijec[%d] = %s\n", broj, p[broj]);

            /* Na kraju, povecaj broj rijeci. */
        ++broj;
    }
    broj_rijeci = broj;
    return status;
}

  /* Sortiranje rjecnika izborom ekstrema.
     Dovodimo najmanju rijec na pocetak.
     Koristimo samo zamjene pokazivaca. */

  /* Polje pokazivaca na znakove-stringove. */

void sort(char *p[], int n)
{
    int i, j, ind_min;
    char *temp;

    for (i = 0; i < n - 1; ++i) {
        ind_min = i;
        for (j = i + 1; j < n; ++j)
            if (strcmp(p[j], p[ind_min]) < 0)
                ind_min = j;
        if (i != ind_min) {
            temp = p[i];
            p[i] = p[ind_min];
            p[ind_min] = temp;
        }
    }

    return;
}

  /* Ispisuje sve rijeci u rjecniku. */

void ispis(char *p[])
{
    int i;

    for (i = 0; i < broj_rijeci; ++i)
        puts(p[i]);

    return;
}

  /* Glavni program. */

int main(void)
{
    int status;

    status = unos();
    if (status == -1)
        printf("Greska: EOF u citanju rijeci.\n");
    else if (status == 1)
        printf("Greska: Realokacija nije uspjela.\n");

        /* Bez obzira na eventualnu gresku, sortiramo
           sve uspjesno ucitane rijeci. Zato nema else. */

    printf("Broj rijeci = %d\n", broj_rijeci);
    printf("\nPolazni rjecnik:\n");
    ispis(p);
    sort(p, broj_rijeci);
    printf("\nSortirani rjecnik:\n");
    ispis(p);

        /* Oslobodi svu zauzetu memoriju.
           Ovo, zbog stalne realokacije polja w i p,
           radi i kad dodje do greske u citanju. */
    free(w);
    free(p);

    return 0;
}
