/* ffun_lib.h --- Sortiranje binarnih (neformatiranih) datoteka.

   Datoteka zaglavlja sadrzi sljedece deklaracije:
   - tip objekata u datoteci, njegovu duljinu i funkciju za usporedjivanje,
   - osnovne funkcije za operacije s datotekom, uz kontrolu greske,
   - genericke funkcije za sortiranje, simulacijom rada na polju
     (indeksi su tipa long int, kao kod ftell, fseek).

   Implementacija je u datoteci ffun_lib.c.
*/

/* ======================================================================== */

    /* Tip objekata u datoteci je obj_type = int,
       a velicina takvog objekta je obj_size. */

typedef int obj_type;

size_t obj_size;    /* Definicija je u ffun_lib.c */

/* ======================================================================== */

    /* Funkcija za usporedjivanje dva objekta. Objekti su zadani
       pokazivacima na njih, a rezultat je (kao kod strcmp):
       < 0  <=> prvi je < drugog,  = 0  <=> prvi == drugi,
       > 0  <=> prvi je > drugog. */

    /* U nasem slucaju, obj_cmp je usporedba cijelih brojeva.
       Uocite da else-ovi nisu potrebni. */

int obj_cmp(const obj_type *p_a, const obj_type *p_b);

/* ======================================================================== */

    /***** Osnovne funkcije na datoteci s kontrolom greske. *****/

    /* Postavljanje trazene pozicije u datoteci (funkcijom fseek)
       uz kontrolu greske. */

void postavi_fpos(FILE *f_obj, long int offset, int origin);

    /* Broj objekata u datoteci, s provjerom duljine datoteke i
       kontrolom je li duljina cjelobrojni visekratnik duljine objekta.
       U slucaju greske, funkcija prekida izvrsavanje programa.
       Izlazni tip je long int, a ne size_t, tako da za praznu datoteku
       (n_obj = 0) smijemo koristiti n_obj - 1 u istom tipu. */

long int broj_objekata(FILE *f_obj, size_t obj_size);

    /* Praznjenje/pisanje spremnika u datoteku (funkcijom fflush)
       nakon pisanja (pri prijelazu s pisanja na citanje),
       uz kontrolu greske. */

void isprazni_spremnik(FILE *f_obj);

/* ======================================================================== */

    /* Citanje jednog objekta s trenutne pozicije u datoteci. */

void ucitaj_obj(FILE *f_obj, obj_type *p_x, size_t obj_size);

    /* Pisanje jednog objekta na trenutnu poziciju u datoteci. */

void napisi_obj(FILE *f_obj, obj_type *p_x, size_t obj_size);

/* ======================================================================== */

    /***** Genericke funkcije na datoteci --- simulacija polja. *****/

    /* Citanje i-tog objekta x[i] iz datoteke (i pocinje od 0).
       Indeksi su tipa long int, kao kod ftell, fseek. */

void ucitaj_obj_i(FILE *f_obj,
                  long int i, obj_type *p_x, size_t obj_size);

    /* Pisanje i-tog objekta x[i] u datoteku (i pocinje od 0).
       Indeksi su tipa long int, kao kod ftell, fseek. */

void napisi_obj_i(FILE *f_obj,
                  long int i, obj_type *p_x, size_t obj_size);

    /* Zamjena i-tog i j-tog objekta u datoteci.
       Indeksi su tipa long int, kao kod ftell, fseek. */

void zamjena_obj_ij(FILE *f_obj,
                    long int i, long int j, size_t obj_size);

/* ======================================================================== */

    /* Ispis sadrzaja datoteke objekata. */

void ispisi_dat(FILE *f_obj, size_t obj_size);

/* ======================================================================== */

    /* Provjera uzlazne sortiranosti datoteke objekata. */

int provjeri_sort_dat(FILE *f_obj, size_t obj_size);

/* ======================================================================== */
